/*
 * Copyright (c) 2010 Moreno Marzolla
 * Copyright (c) 2003-2005 The BISON Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package example.vman;

import peersim.config.Configuration;
import peersim.config.FastConfig;
import peersim.core.*;
import peersim.vector.SingleValueHolder;
import peersim.cdsim.CDProtocol;

/**
 * <p>
 * This class implements a simple consolidation strategy;
 * a computing node with fewer VMs sends some of them to a neighbor
 * with more VMs. In this way we obtain the situation in which few
 * nodes are fully loaded, while others are empty.
 * </p>
 * <p>
 * The class subclasses {@link peersim.vector.SingleValueHolder} in order to be
 * type compatible with its observers and initializers object companions.
 * </p>
 * 
 */
public class BasicConsolidation extends SingleValueHolder implements CDProtocol {

    // ------------------------------------------------------------------------
    // Parameters
    // ------------------------------------------------------------------------

    /**
     * Node capacity. The capacity is the maximum number of
     * Virtual Machines that a node can host. Defaults to 1.
     * 
     * @config
     */
    protected static final String PAR_CAPACITY = "capacity";

    // ------------------------------------------------------------------------
    // Fields
    // ------------------------------------------------------------------------

    /** Capacity. Obtained from config property {@link #PAR_CAPACITY}. */
    private final int capacity_value;
    
    // ------------------------------------------------------------------------
    // Initialization
    // ------------------------------------------------------------------------
    /**
     * Standard constructor that reads the configuration parameters. Invoked by
     * the simulation engine.
     * 
     * @param prefix
     *            the configuration prefix for this class.
     */
    public BasicConsolidation(String prefix) {
        super(prefix);
        // get capacity value from the config file. Default 1.
	capacity_value = (Configuration.getInt(prefix + "." + PAR_CAPACITY, 1));
    }

    // The clone() method is inherited.

    // ------------------------------------------------------------------------
    // Methods
    // ------------------------------------------------------------------------

    /**
     * Returns the capacity of this node
     */
    public int getCapacity() { return capacity_value; };

    /**
     * Using an underlying {@link Linkable} protocol
     * performs a consolidation step with all neighbors of the node
     * passed as parameter.
     * 
     * @param node
     *            the node on which this component is run.
     * @param protocolID
     *            the id of this protocol in the protocol array.
     */
    public void nextCycle(Node node, int protocolID) {
        int linkableID = FastConfig.getLinkable(protocolID);
        Linkable linkable = (Linkable) node.getProtocol(linkableID);

        for (int i = 0; i < linkable.degree(); ++i) {
            Node peer = linkable.getNeighbor(i);
            // The selected peer could be inactive
            if (!peer.isUp())
                continue;
	    BasicConsolidation n = (BasicConsolidation) peer.getProtocol(protocolID);
	    doTransfer(n);
        }
    }

    /**
     * Performs the actual consolidation selecting to make a PUSH or PULL
     * approach. The idea is to send the maximum number of VMs from
     * the node with fewer VMs to the other one. 
     * 
     * @param neighbor
     *            the selected node to talk with.
     */
    protected void doTransfer(BasicConsolidation neighbor) {
        int a1 = (int)this.value;
        int a2 = (int)neighbor.value;
	if ( a1 == 0 || a2 == 0 )
	    return; // if one of the nodes has no VMs, return
	int a1_avail = capacity_value - a1;
	int a2_avail = neighbor.capacity_value - a2;
        int trans = Math.min( Math.min(a1,a2), Math.min(a1_avail, a2_avail));

        if (a1 <= a2) { 
	    // PUSH phase        
            a1 -= trans;
            a2 += trans;
        } else { 
	    // PULL phase
            a1 += trans;
            a2 -= trans;
        }
	assert( a1 >= 0 && a1 <= capacity_value );
	assert( a2 >= 0 && a1 <= capacity_value );
        this.value = (float)a1;
        neighbor.value = (float)a2;
    }
}
