/*
 * Copyright (c) 2010 Moreno Marzolla
 * Copyright (c) 2003-2005 The BISON Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * This class was peersim/examples/loadBalancer; it has been modified
 * in such a way that each node is initialized with a random integer
 * value uniformly distributed on a user-defined interval [min, max].
 * min and max are user-defined parameters.
 */
package example.vman;

import peersim.config.*;
import peersim.core.*;
import peersim.dynamics.NodeInitializer;
import peersim.vector.SingleValue;

/**
 * This class provides a random distribution in a bounded interval
 * defined by {@link #PAR_MIN} and {@link #PAR_MAX} parameters. It is
 * assumed that the network nodes comply to the {@link
 * peersim.vector.SingleValue} interface. This class also implements
 * the {@link NodeInitializer} interface, so that it can be used
 * to initialize single nodes. 
 */
public class RandomDistributionInitializer implements Control, NodeInitializer {

    // ------------------------------------------------------------------------
    // Parameters
    // ------------------------------------------------------------------------

    /**
     * The upper bound of the values.
     * 
     * @config
     */
    private static final String PAR_MAX = "max";

    /**
     * The lower bound of the values. Defaults to -max.
     * 
     * @config
     */
    private static final String PAR_MIN = "min";

    /**
     * The protocol to operate on.
     * 
     * @config
     */
    private static final String PAR_PROT = "protocol";

    // ------------------------------------------------------------------------
    // Fields
    // ------------------------------------------------------------------------
    
    /** Maximum interval value,
    obtained from config property {@link #PAR_MAX}. */
    private final int max;

    /** Manimum interval value,
    obtained from config property {@link #PAR_MIN}. */
    private final int min;

    /** Protocol identifier, obtained from config property {@link #PAR_PROT}. */
    private final int protocolID;

    // ------------------------------------------------------------------------
    // Initialization
    // ------------------------------------------------------------------------
    /**
     * Standard constructor that reads the configuration parameters. Invoked by
     * the simulation engine.
     * 
     * @param prefix
     *            the configuration prefix for this class.
     */
    public RandomDistributionInitializer(String prefix) {
        max = Configuration.getInt(prefix + "." + PAR_MAX);
        min = Configuration.getInt(prefix + "." + PAR_MIN, 0);
        protocolID = Configuration.getPid(prefix + "." + PAR_PROT);
    }

    // ------------------------------------------------------------------------
    // Methods
    // ------------------------------------------------------------------------

    /**
     * This class provides a simple random distribution in a bounded
     * interval defined by parameters {@link #PAR_MIN} and {@link
     * #PAR_MAX}.
     */
    public boolean execute() {
	int tmp;
        for (int i = 0; i < Network.size(); ++i) {
	    initialize( Network.get(i) );
        }
        return false;
    }

    /**
     * Initialize a single node by allocating a random number of virtual
     * machines,
     * defined by parameters {@link #PAR_MIN} and {@link #PAR_MAX}.
     */
    public void initialize(Node n) {
	int tmp = min + CommonState.r.nextInt( max-min+1 );
	assert( tmp >= min );
	assert( tmp <= max );
	((SingleValue) n.getProtocol(protocolID)).setValue(tmp);
    }

}
