/*
 * Copyright (c) 2010 Moreno Marzolla
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package example.vman;

import peersim.config.Configuration;
import peersim.config.FastConfig;
import peersim.core.*;

/**
 * This class implements a simple dynamics of VMs in the system. Each
 * time this control is invoked, a random number of VMs is added to
 * the system at random nodes. The number of VMs to add is uniformly
 * distributed between the {@link PAR_MIN} and {@link PAR_MAX}
 * configuration parameters. Note that a negativa value can be
 * generated (if {@link PAR_MIN} is less than zero): in this case, the
 * corresponding number of VMs is removed from the system.
 *
 * This class is smart enough to check that capacity constraints are
 * not violated (i.e., that we are not trying to remove more VMs
 * than those actually running, or that we are not trying to add more
 * VMs than the residual system capacity).
 *
 * At each time step, this class prints to standard output the number
 * of VMs which have been added (which can be either a positive or negative
 * integer).
 */
public class VMDynamics implements Control {

    // ------------------------------------------------------------------------
    // Parameters
    // ------------------------------------------------------------------------

    /**
     * Minimum number of VMs to remove; can be either a positive or
     * negative integer
     */
    protected static final String PAR_MIN = "min";

    /**
     * Maximum number of VMs to remove; can be either a positive or negative
     * integer.
     */
    protected static final String PAR_MAX = "max";

    /**
     * This parameter indicates the BasicConsolidation protocol
     * this class operates on.
     */
    protected static final String PAR_PROT = "protocol";

    // ------------------------------------------------------------------------
    // Fields
    // ------------------------------------------------------------------------
    private final int min;
    private final int max;
    private final int protocolID;
    private final String name;

    // ------------------------------------------------------------------------
    // Initialization
    // ------------------------------------------------------------------------
    public VMDynamics(String name) {
	this.name = name;
        max = Configuration.getInt(name + "." + PAR_MAX);
        min = Configuration.getInt(name + "." + PAR_MIN, -max);
        protocolID = Configuration.getPid(name + "." + PAR_PROT);
    }

    // ------------------------------------------------------------------------
    // Methods
    // ------------------------------------------------------------------------
    public boolean execute() 
    {
	int i;
        long time = peersim.core.CommonState.getTime();
	// Compute the number of running VMs, and the total system
	// capacity. This is done to ensure that capacity constraints
	// are not violated
	int nvm = 0;
	int cap = 0;
	for (i=0; i<Network.size(); ++i) {
	    BasicConsolidation n = (BasicConsolidation)Network.get(i).getProtocol(protocolID);
	    nvm += n.getValue();
	    cap += n.getCapacity();
	}
	// Compute the minimum and maximum number of VMs which can be
	// added (removed) from the system without violating the
	// capacity constraints.
	int mmin = Math.max(min, -nvm);
	int mmax = Math.min(max,cap-nvm);
	int nval = mmin + CommonState.r.nextInt( mmax-mmin+1 ); // number of VMs to add/remove
	// Sanity check
	assert( nval >= mmin );
	assert( nval <= mmax );
	System.out.println(name+": "+time+" "+nval);
	// Add/remove "nval" VMs from the system
	for ( i=0; i<Math.abs(nval); ) {
	    // get a random node n
	    int node_id = CommonState.r.nextInt(Network.size());
	    BasicConsolidation n = (BasicConsolidation)Network.get(node_id).getProtocol(protocolID);
	    if ( nval>0 && n.getValue() < n.getCapacity() ) { // add VMs
		n.setValue( n.getValue()+1 );
		++i;
	    } else if ( nval<0 && n.getValue() > 0 ) { // remove VMs
		n.setValue( n.getValue()-1 );
		++i;
	    }
	}
	return false;
    }
}
